/* -*- mode: C++; c-basic-offset: 4  -*- */
/* ex: set shiftwidth=4 tabstop=4 expandtab: */
/*
 * Copyright (c) 2010-2011, Georgia Tech Research Corporation
 * All rights reserved.
 *
 * Author(s): Neil T. Dantam <ntd@gatech.edu>
 * Georgia Tech Humanoid Robotics Lab
 * Under Direction of Prof. Mike Stilman <mstilman@cc.gatech.edu>
 *
 *
 * This file is provided under the following "BSD-style" License:
 *
 *
 *   Redistribution and use in source and binary forms, with or
 *   without modification, are permitted provided that the following
 *   conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *   CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 *   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 *   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *   ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *   POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef AMINO_HPP
#define AMINO_HPP
/**
 * \file amino.hpp
 * \author Neil T. Dantam
 */

#include "amino.h"
#include "amino/mem.hpp"
#include "amino/tf.hpp"


//namespace amino {
//#include "amino/mat.hpp"
//}

/// amino namespace
namespace amino {

/// Locks mutex on construction, unlocks on destruction
class ScopedMutex {
public:
    /// costruct this and lock m
    ScopedMutex( pthread_mutex_t *m ) :
    mutex(m)
    {
        pthread_mutex_lock(mutex);
    }
    /// costruct this and lock m
    ScopedMutex( pthread_mutex_t &m ) :
    mutex(&m)
    {
        pthread_mutex_lock(mutex);
    }
    /// destroy this and unlock m
    ~ScopedMutex() {
        pthread_mutex_unlock(mutex);
    }

private:
    /// the mutex
    pthread_mutex_t *mutex;
};



template<typename T>
T next( T p )
{
    T q = p;
    q++;
    return q;
}


template<typename T>
T prev( T p )
{
    T q = p;
    q--;
    return q;
}

}

#endif //AMINO_HPP
